@@ -2,18 +2,20 @@ from __future__ import annotations
 from pathlib import PosixPath, WindowsPath, _NormalAccessor, \
   Path, PurePath, _ignore_error
 from typing import Optional, List, Union, AsyncIterable
-from os import stat_result, PathLike
+from os import stat_result
 from stat import S_ISDIR, S_ISLNK, S_ISREG, S_ISSOCK, S_ISBLK, \
   S_ISCHR, S_ISFIFO
 import os
 
-from anyio import open_file
+from aiofile import AIOFile
+from aiofiles import os as async_os
+from aiofiles.os import wrap as method_as_method_coro, \
+  wrap as func_as_corofunc
 
 from .selectors import _make_selector
 from .flavours import _async_windows_flavour, _async_posix_flavour
-from .wrap import coro_as_method_coro, func_as_method_coro, to_thread, \
-  method_as_method_coro, func_to_async_func
-from .handle import IterableAIOFile, get_handle
+from .wrap import coro_as_method_coro, func_as_method_coro, to_thread
+from .handle import IterableAIOFile, read_full_file
 from .scandir import EntryWrapper, scandir_async
 from .types import Final, Literal, FileMode
 
@@ -23,14 +25,12 @@ ON_ERRORS: Final[str] = 'ignore'
 NEWLINE: Final[str] = '\n'
 
 
-Paths = Union[Path, PathLike, str]
-
-
-close = func_to_async_func(os.close)
+getcwd = func_as_corofunc(os.getcwd)
+close = func_as_corofunc(os.close)
 
 
 class _AsyncAccessor(_NormalAccessor):
-  stat = func_as_method_coro(os.stat)
+  stat = coro_as_method_coro(async_os.stat)
   lstat = func_as_method_coro(os.lstat)
   open = func_as_method_coro(os.open)
   listdir = func_as_method_coro(os.listdir)
@@ -39,14 +39,14 @@ class _AsyncAccessor(_NormalAccessor):
   if hasattr(_NormalAccessor, 'lchmod'):
     lchmod = method_as_method_coro(_NormalAccessor.lchmod)
 
-  mkdir = func_as_method_coro(os.mkdir)
+  mkdir = coro_as_method_coro(async_os.mkdir)
   unlink = func_as_method_coro(os.unlink)
 
   if hasattr(_NormalAccessor, 'link'):
     link = method_as_method_coro(_NormalAccessor.link)
 
-  rmdir = func_as_method_coro(os.rmdir)
-  rename = func_as_method_coro(os.rename)
+  rmdir = coro_as_method_coro(async_os.rmdir)
+  rename = coro_as_method_coro(async_os.rename)
   replace = func_as_method_coro(os.replace)
 
   symlink = staticmethod(
@@ -55,7 +55,7 @@ class _AsyncAccessor(_NormalAccessor):
 
   utime = func_as_method_coro(os.utime)
   readlink = method_as_method_coro(_NormalAccessor.readlink)
-  remove = func_as_method_coro(os.remove)
+  remove = coro_as_method_coro(async_os.remove)
 
   async def owner(self, path: str) -> str:
     try:
@@ -81,6 +81,8 @@ class _AsyncAccessor(_NormalAccessor):
     async for entry in scandir_async(*args, **kwargs):
       yield entry
 
+  expanduser = staticmethod(func_as_corofunc(os.path.expanduser))
+
 
 _async_accessor = _AsyncAccessor()
 
@@ -125,7 +127,11 @@ class AsyncPath(Path, AsyncPurePath):
     if cls is AsyncPath:
       cls = AsyncWindowsPath if os.name == 'nt' else AsyncPosixPath
 
-    self = cls._from_parts(args, init=False)
+    try:
+      self = cls._from_parts(args, init=False)
+
+    except TypeError:
+      self = cls._from_parts(args)
 
     if not self._flavour.is_supported:
       name: str = cls.__name__
@@ -146,7 +152,7 @@ class AsyncPath(Path, AsyncPurePath):
     errors: Optional[str] = ON_ERRORS,
     newline: Optional[str] = NEWLINE,
   ) -> IterableAIOFile:
-    return get_handle(
+    return IterableAIOFile(
       self._path,
       mode,
       encoding=encoding,
@@ -156,11 +162,12 @@ class AsyncPath(Path, AsyncPurePath):
 
   async def read_text(
     self,
-    encoding: str | None = DEFAULT_ENCODING,
-    errors: str | None = ON_ERRORS
+    encoding: Optional[str] = DEFAULT_ENCODING,
+    errors: Optional[str] = ON_ERRORS
   ) -> str:
+
     async with self.open('r', encoding=encoding, errors=errors) as file:
-      return await file.read()
+      return await file.read_text(encoding=encoding, errors=errors)
 
     #path = str(await self.resolve())
 
@@ -370,11 +377,11 @@ class AsyncPath(Path, AsyncPurePath):
       #return False
 
   @classmethod
-  def cwd(cls: type) -> str:
+  async def cwd(cls: type) -> str:
     """Return a new path pointing to the current working directory
     (as returned by os.getcwd()).
     """
-    cwd: str = os.getcwd()
+    cwd: str = await getcwd()
     return cls(cwd)
 
   @classmethod
@@ -382,21 +389,12 @@ class AsyncPath(Path, AsyncPurePath):
     """Return a new path pointing to the user's home directory (as
     returned by os.path.expanduser('~')).
     """
-    coro = cls()._flavour.gethomedir(None)
-    homedir: str = await coro
+    return await cls('~').expanduser()
 
-    return cls(homedir)
-
-  async def samefile(
-    self,
-    other_path: Union[AsyncPath, Paths]
-  ) -> bool:
+  async def samefile(self, other_path: Union[AsyncPath, Path]) -> bool:
     """Return whether other_path is the same or not as this file
     (as returned by os.path.samefile()).
     """
-    if isinstance(other_path, Paths.__args__):
-      other_path = AsyncPath(other_path)
-
     if isinstance(other_path, AsyncPath):
       try:
         other_st = await other_path.stat()
@@ -461,7 +459,7 @@ class AsyncPath(Path, AsyncPurePath):
     async for p in selector.select_from(self):
       yield p
 
-  def absolute(self) -> AsyncPath:
+  async def absolute(self) -> AsyncPath:
     """Return an absolute version of this path.  This function works
     even if the path doesn't point to anything.
     No normalization is done, i.e. all '.' and '..' will be kept along.
@@ -472,7 +470,7 @@ class AsyncPath(Path, AsyncPurePath):
         return self
     # FIXME this must defer to the specific flavour (and, under Windows,
     # use nt._getfullpathname())
-    obj = self._from_parts([os.getcwd()] + self._parts, init=False)
+    obj = self._from_parts([await getcwd()] + self._parts, init=False)
     obj._init(template=self)
     return obj
 
@@ -684,9 +682,10 @@ class AsyncPath(Path, AsyncPurePath):
     (as returned by os.path.expanduser)
     """
     if (not (self._drv or self._root) and
-      self._parts and self._parts[0][:1] == '~'
-    ):
-      homedir = await self._flavour.gethomedir(self._parts[0][1:])
+      self._parts and self._parts[0][:1] == '~'):
+      homedir = await self._accessor.expanduser(self._parts[0])
+      if homedir[:1] == "~":
+        raise RuntimeError("Could not determine home directory.")
       return self._from_parts([homedir] + self._parts[1:])
 
     return self
