@@ -1,8 +1,16 @@
-from typing import AsyncIterable, Iterable, Any, List
+from typing import Callable, Awaitable, AsyncIterable, Iterable, \
+  Any, List
 from os import scandir, DirEntry, stat_result
+from dataclasses import dataclass
 
 from .wrap import to_thread
 
+try:
+  from asyncio import to_thread
+
+except ImportError:
+  pass
+
 
 class EntryWrapper:
   __slots__ = "entry",
